/************************************************************

  MD_FCT.h

  Definitions of functions found in MD_FCT.c and
  MD_FCT_utils.c

  Created by Edward A. Richley (richley@mailaps.org)

  This file is subject to the terms and conditions of the 
  GNU General Public License. 


************************************************************/
#ifndef MD_FCT_flag
#define MD_FCT_flag

/************************************************************

  Globals used by MD_FCT which are accessible from user code

  MD_Source_dt is a timestep constraint which can be set
  from the user-defined "vel" procedure. 
  MD_Total_time is the total time into the simulation.
  MD_Courant is the Courant condition.
  DUMMY_GRID is a pointer to a zero-dimensional grid
  with one data point.

************************************************************/
extern double MD_Source_dt, MD_Total_time, MD_Courant;
extern double MD_Courant_retry_factor, MD_Courant_update_factor, MD_FCT_courant_tol;
extern dblptr MD_FCT_rho_bc;
extern geometry_ptr DUMMY_GRID;
extern bool MD_trace;
extern MD_FCT_species_ptr MD_species;
extern geometry_ptr MD_grid;
extern MD_STEP MD_current_step;
/************************************************************

  MD_Flow(set, vel, Total_Time)

  Main routine to convect a species set "set" for time "Total_Time"
  with user-supplied source/velocity definition procedure "vel".

  if vel returns FALSE, a timestep correction takes place.

************************************************************/
extern bool MD_Flow(MD_FCT_species_set_ptr, bool(), double);
/************************************************************

  MD_FCT_new_grid(name)

  Makes a new grid named "name". Returns a pointer to it.

************************************************************/
extern geometry_ptr MD_FCT_new_grid();
/************************************************************

  MD_FCT(spe, dt )

  One step (of length "dt") of FCT convection for one species, "spe".  
  Greatest epsilon is returned.

************************************************************/
extern double MD_FCT();
/************************************************************

  MD_Diffusion

  Implicit, one-dimensional diffusion step for implementing
  parabolic terms of the form:

  d (rho)
  ______  = div( D * grad(kappa*rho))
   dt

************************************************************/
extern double MD_Diffusion();
/************************************************************

  MD_tri_diag(A, u, r, N)

  tri-diagonal matrix solver for Au = r

************************************************************/
extern bool MD_tri_diag();
/************************************************************

  MD_chemstep(rlist, splist, dt, first)

  Evaluates and updates all reactions in rlist.  Each reaction is
  advanced on its own grid (the grid associated with the reaction) for
  all species in splist which are defined on that grid.

  Explicit first order evaluation is performed, but
  is converted to second order automatically when used
  with MD_Richardson. 

  dt is the time step.

  If first == true, takes rho->n to rho->np1. Otherwise,
  takes rho->np1 to rho->np1. 

  Makes assumptions about grids being all the same for
  all species. Covers all of space.

  returns maximum relative change.

************************************************************/
double MD_chemstep();

#endif
