/************************************************************

  MD_energy_axis.c

  Axis definition routines for kinetic energy axis.

  Created by Edward A. Richley (richley@mailaps.org)

  This file is subject to the terms and conditions of the 
  GNU General Public License. 

************************************************************/
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>

#ifdef ccc
#include <cpml.h>
#endif
#include "basics.h"
#include "MD_FCT_defs.h"

static unsigned int Nfac = 10;
/************************************************************

  MD_energy_axis()

  Axis initialization routine. 

  Uneven grid spacing is used:


  |.|.|.| .| .| . | . |  . |  . |  .  |  .  |   .  |   .   |
  |                                                        |
 u=0                                                     u=uMAX

  according to:
                    2
  du = du0 + alpha*i

  and it all sums up to uMAX.

  du0 is set to uMAX/(Nu*Nfac) where Nfac is some constant
  (probably 10).


************************************************************/
void MD_energy_axis(ax, grid, pos)
axis_ptr ax;
geometry_ptr grid;
unsigned int pos;
{
  unsigned int i, j;
  unsigned long int l, Nu;
  double u, du, du0, eMAX, alpha, sum, u_int, s;

  Nu = (unsigned long int)(ax->num_pts);
  j = ax->index;

  eMAX = ax->length;
  du0 = eMAX/((double)(Nfac*Nu));
  sum = (double)(Nu*(Nu+1))*(double)(2*Nu+1)/6.0;
  alpha = (eMAX*((double)(Nfac - 1)/(double)Nfac))/sum;
  l = (unsigned long int)((j <= 1)?0:(j-1));
  s = (double)(l*(l+1))*(double)(2*l+1)/6.0;
  i = (j!= 0)?j:1;
  du = du0 + alpha*(double)(i*i);
  u_int = (j == 0)?(-du):((double)l*du0+alpha*s);
  u = u_int + .5*du;
  ax->r[pos] = u;
  ax->Area[pos] = 1.0;
  ax->Volume[pos] = du;
  return;
}
