/************************************************************

  MD_plplot


  Routines for starting plplot, and for plotting data.

  Created by Edward A. Richley (richley@mailaps.org)

  This file is subject to the terms and conditions of the 
  GNU General Public License. 


************************************************************/
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include "basics.h"
#include "MD_plplot.h"
#include <sys/types.h>
#include <sys/stat.h>
#define PL_DOUBLE
#include <plplot/plplot.h>
#undef pi
#include "MD_FCT_defs.h"
#include "MD_FCT.h"

#define NP 400
#define NCMD 200
#define X_AXIS_LABEL_SIZE 50
#define Y_AXIS_LABEL_SIZE 50
#define Z_AXIS_LABEL_SIZE 50

enum N_dimensions {zero, one, two, three, four, many};
typedef enum N_dimensions N_axes;
string PSDIR = NULL;

extern char **environ;
PL_color MD_axis_color = yellow, MD_label_color = white;
static PL_color axis_clr = yellow, label_clr = white;
static char p[NP];
static unsigned int Ncolor = 15, strmcnt;
static double y2d[3] = {0, 1, 2};
static string option;
static string DefaultViewStr3d = "[30 30]", ViewStr3d;
bool MD_plplot_hold = false, MD_plplot_append = true;
static bool XWIN_FLAG, append;
static char xlabelstr[X_AXIS_LABEL_SIZE];
static char ylabelstr[Y_AXIS_LABEL_SIZE];
static char zlabelstr[Z_AXIS_LABEL_SIZE];
/************************************************************

  plot_array()

  Plot array A. If it's really a 2d plot, do a 2d plot.
  Plot made to the current stream.

************************************************************/
void plot_array(Z, x, y, N, M, xlabel, ylabel, zlabel, title, clr, space, sp)
dblptr x, y;
const double * const *Z;
unsigned int N, M;
PL_color clr;
string xlabel, ylabel, zlabel, title;
N_axes space;
MD_FCT_species_ptr sp;
{
PLFLT xmin, xmax, ymin, ymax, zmin, zmax, zt, az, alt;
int i, j;
char ch, ViewStr[100];


if (space == two) {
  xmin = x[0]; xmax = x[N-1];
  zmin = zmax = Z[0][0];
  for(i=0; i<N; i++){
    zt = Z[0][i];
    zmin = (zt < zmin)?zt:zmin;
    zmax = (zt > zmax)?zt:zmax;
  }
  if (!(zmax > zmin)){
    zmax = 1.1*zmin;
    if (!(zmax > zmin)){
      zmax = 1.0;
    }
  }
  pladv(0);
  plfont(2);
  plvpor(0.15, 0.95, 0.1, 0.85);
  plwind(xmin, xmax, zmin, zmax); 
  plcol0(axis_clr);
  plbox("bcnst", 0.0, 0, "bcnst", 0.0, 0);
  plcol0(clr);
  plline(N, x, Z[0]);
  plcol0(label_clr);
  plmtex("b", 3.2, 0.5, 0.5, xlabel);
  plmtex("l", 4.0, 0.9, 1.0, zlabel);
  plmtex("t", 2.0, 0.5, 0.5, title);
}
else if (space == three) {
  ViewStr3d = (strcmp(sp->ViewAngle,""))?sp->ViewAngle:DefaultViewStr3d;
  strcpy(ViewStr, ViewStr3d);
  for(i=0; i<strlen(ViewStr3d); i++){
   ch = ViewStr[i];
   if ((ch == '[')||(ch == ',')||(ch == ']')) {ch = ' ';}
   ViewStr[i] = ch;
  }
  sscanf(ViewStr,"%lf %lf",&az, &alt);
  alt = (alt < 0)?0:(alt > 90.0)?90.0:alt;
  xmin = x[0]; xmax = x[N-1];
  ymin = y[0]; ymax = y[M-1];
  zmin = zmax = Z[0][0];
  for(j=0; j<M; j++){
    for(i=0; i<N; i++){
      zt = Z[j][i];
      zmin = (zt < zmin)?zt:zmin;
      zmax = (zt > zmax)?zt:zmax;
    }
  }
  if (!(zmax > zmin)){
    zmax = 1.1*zmin;
    if (!(zmax > zmin)){
      zmax = 1.0;
    }
  }
  pladv(0);
  plfont(2);
  plvpor(0.0, 1.0, 0.0, 0.8);
  plwind(-1.0, 1.0, -1.0, 1.5);
  plw3d(1.0, 1.0, 1.2, ymin, ymax, xmin, xmax, zmin, zmax, alt, az);
  plcol0(axis_clr);
  plbox3("bnstu", ylabel, 0.0, 0,
	 "bnstu", xlabel, 0.0, 0,
	 "bcdmnstuv", zlabel, 0.0, 4);
  plcol0(clr);
  plmesh(y, x, Z, M, N, 3);
  plcol0(label_clr);
  plmtex("t", 1.0, 0.5, 0.5, title);
}
return;
}
/************************************************************

  plotrho()

  Tries to plot sp->rho0 on the current figure. 
  If it's really a 2d plot (one independent axis) 
  an array is formed with a fake 2nd independent
  axis (with only two grid points) so that
  plot_array can handle it.

  Also creates PostScript output in file
  /PSDIR/sp->name.ps where /PSDIR is defined 
  from either the environment variable "PSDIR", 
  or created from the environment variable "HOME",
  as $HOME/ps/.
  If MD_plplot_append is false, 
  file is always overwritten. If MD_plplot_append
  is true, file is overwritten the first time,
  and appended threafter. 

************************************************************/
void plotrho(sp,clr)
MD_FCT_species_ptr sp;
PL_color clr;
{
static char  p[100];
static string cmdstr=NULL, flname=NULL;
static int nx = 0, nnx = 0, lastN = 0, lastM = 0;
string xlabel, ylabel, zlabel;
unsigned int N, M, i, j, pos;
axis_ptr axs, axx, axy;
dblptr x, y, A;
static bool First_Time = true;
static dblptr *Z = NULL, X = NULL, Y = NULL;
double A00, A0N, AM0, AMN;
bool first_flag;
int R, G, B;
N_axes space;

if (First_Time){
  First_Time = false;
  cmdstr = get_memory(cmdstr, NCMD, char);
  flname = get_memory(flname, NCMD, char);
}
x = y = NULL;
nx = N = M = 1;
space = zero;
for(axs = (sp->grid->axes), i = 0; axs!=NULL; axs = (axs->next), i++){
  if (i == 0) {
    axx = axs;
    N = axs->num_pts + 2;
    x = axs->r;
    if (
	((strlen(axs->name)+strlen(axs->units)+3)<=X_AXIS_LABEL_SIZE)&&
	(strlen(axs->units) != 0)
	){
      xlabel = xlabelstr;
      xlabel = strcpy(xlabel,axs->name);
      xlabel = strcat(xlabel," (");
      xlabel = strcat(xlabel, axs->units);
      xlabel = strcat(xlabel,")");
    } else {
      xlabel = axs->name;
    }
    space = two;
    if (N > lastN) {
      X = get_memory(X, N, double);
      lastN = N;
    }
  } else if (i == 1){
    axy = axs;
    M = axs->num_pts + 2;
    y = axs->r;
    if (
	((strlen(axs->name)+strlen(axs->units)+3)<=Y_AXIS_LABEL_SIZE)&&
	(strlen(axs->units) != 0)
	){
      ylabel = ylabelstr;
      ylabel = strcpy(ylabelstr,axs->name);
      ylabel = strcat(ylabel," (");
      ylabel = strcat(ylabel, axs->units);
      ylabel = strcat(ylabel,")");
    } else {
      ylabel = axs->name;
    }
    space = three;
    nx = (M > nx)?M:nx;
    if (M > lastM) {
      Y = get_memory(Y, M, double);
      lastM = M;
    }
  } else {
    space = many;
    printf("Too many axes to plot %s\n",sp->name);
    return;
  }
}
if (nx > nnx) {
  Z = get_memory(Z, nx, dblptr);
  nnx = nx;
}

fprintf(stderr,"plotting in %u-space\n",space);
A = sp->rho0;

strcpy(cmdstr, sp->name);
strcat(cmdstr," at time: ");
sprintf(cmdstr,"%s %8.5e",cmdstr,MD_Total_time);

 if (
     ((strlen(sp->name)+strlen(sp->units)+3)<=Z_AXIS_LABEL_SIZE)&&
     (strlen(sp->units) != 0)
     ){
   zlabel = zlabelstr;
   zlabel = strcpy(zlabel,sp->name);
   zlabel = strcat(zlabel," (");
   zlabel = strcat(zlabel, sp->units);
   zlabel = strcat(zlabel,")");
 } else {
   zlabel = sp->units;
 }

/*
  If it's a 2d plot (one independent) just keep going
*/
if (space == two) {
  Z[0] = A;
}
if (space == three){         /* fix up corners (in case data there is bad) */
  A00 = A[0]; A0N = A[N-1]; AM0 = A[N*(M-1)]; AMN = A[N*M-1];
  A[0] = .5*(A[1]+A[N]);
  A[N-1] = .5*(A[N-2]+A[2*N-1]);
  A[N*(M-1)] =.5*(A[N*(M-2)]+A[N*(M-1)+1]);
  A[N*M-1] = .5*(A[N*M-2] + A[N*M - 1 - N]);
  for(j=0; j<M; j++){
    Z[j] = A + j*N;
  }
  for(i=0, pos=0; i<M; i++, pos += axy->mul){
    Y[i] = y[pos];
  }

}
if ((space == two)||(space == three)){
  for(i=0, pos=0; i<N; i++, pos += axx->mul){
    X[i] = x[pos];
  }
  if (sp->Interactive_Plot) { 
    plsstrm(0);
    if (!XWIN_FLAG){
      plsdev("xwin");
      plscolbg(0x00, 0x00, 0x00); /* Set background black */
      plinit();
      XWIN_FLAG = true;  
    }
    axis_clr = MD_axis_color; label_clr = MD_label_color;

    plot_array(Z,X,Y,N,M,xlabel,ylabel,zlabel,cmdstr,clr, space, sp);
  }
  if (sp->plot) {
    first_flag = !append; /* First time needs to re-create the file */

    sprintf(flname,"%s/%s.ps",PSDIR,sp->name);
    if (sp->figure == 0 ) {
      ++strmcnt;
      sp->figure = strmcnt;
      first_flag = true;
    }
    plsstrm(sp->figure);
    if (first_flag){
      plsfnam(flname);
      plsdev("psc");
    }
    plsori(-1);
    if (first_flag){
      plscolbg(0xFF, 0xFF, 0xFF); /* Set background white */
      plinit();
    }
    clr = red; axis_clr = brown; label_clr = blue;
    plot_array(Z,X,Y,N,M,xlabel,ylabel,zlabel,cmdstr,clr, space,sp);
    if (!MD_plplot_append) {plend1();} /* because next time we want to 
overwrite the file */
  }
}
if (space == three){
  A[0] = A00; A[N-1] = A0N; A[N*(M-1)] = AM0; A[N*M-1] = AMN;
}
printf("plot done for %s with color %u\n",sp->name,clr);
return;
}
/************************************************************

  plot_all_rho(set, clr)

  Plot all species in set, with color clr. 
  First, see if PSDIR is already set. If not,
  look for environment variable PSDIR. If not
  there, create one from $HOME/ps. 
  Once PSDIR has been determines, see if it exists.
  If not, try to make it. Exit on failure.

  Check to see if this set has been plotted yet.

************************************************************/
void plot_all_rho(set, clr)
MD_FCT_species_set_ptr set;
PL_color clr;
{
MD_FCT_species_set_rec_ptr  rec;
MD_FCT_species_ptr sp;
static unsigned int next_fig = 1, i;
static bool First_Time = true;
static string cmdstr = NULL, dirstr = NULL, homestr = NULL;
int strpos, status;
struct stat buf;
mode_t mode;


 if (First_Time){
   i = 0;
   while(environ[i] != NULL){
     if (!strncmp(environ[i],"PSDIR",5)){
       dirstr = environ[i]+6;
     }
     if (!strncmp(environ[i],"HOME",4)){
       homestr = environ[i]+5;
     }
     i++;
   }
   if (dirstr == NULL){
     dirstr = get_memory(dirstr,(strlen(homestr)+strlen("/ps")),char);
     dirstr = strcpy(dirstr,homestr);
     dirstr = strcat(dirstr,"/ps");
   } 
   if (PSDIR == NULL) {PSDIR = dirstr;}
   status = stat(PSDIR,&buf);
   if ((!(buf.st_mode & S_IFDIR)) || (status == -1)) {
     mode = S_IRUSR | S_IWUSR | S_IXUSR;
     status = mkdir(PSDIR, mode);
     if (status == -1){
       fprintf(stderr,"Can't find or make output directory for postscript: %s\n",PSDIR);
       fprintf(stderr,"mkdir failed. errno: %i\n", errno);
       exit(1);
     }
   } 
   fprintf(stderr,"Using directory: %s for postscript output\n",PSDIR);
   cmdstr = get_memory(cmdstr, NCMD, char);
   atexit(plend);
   strmcnt = 0;
   XWIN_FLAG = false;
   First_Time = false;
 }

 if (set->Initial_Plot){
   set->Initial_Plot = false;
   append = false;
 } else {
   append = MD_plplot_append;
 }
for(rec = set->start; rec != NULL; rec = rec->next){
  sp = rec->first;
  if (sp->grid->num_dim != 0){
    printf("plotting %s\n",sp->name);
    plotrho(sp,clr);
  }
}
return;
}

#undef NP
#undef NCMD




